#!/usr/bin/env python
import sys, os, shutil, math, commands, glob
from pylab import *
from PyFoam.RunDictionary.BlockMesh import BlockMesh 
from subprocess import Popen
from subprocess import call

# divSchems can be found in the following directories:
#    $WM_PROJECT_DIR/src/finiteVolume/interpolation/surfaceInterpolation/schemes
#    $WM_PROJECT_DIR/src/finiteVolume/interpolation/surfaceInterpolation/limitedSchemes
divSchemes=['QUICK']
timeSteps=['0.05','0.01']
#timeSteps=['0.05']
nCorrectors=['30','80']
#nCorrectors=['30']
relaxationFactors=[['0.8','0.3'],['0.7','0.3']]
#relaxationFactors=[['0.8','0.3']]
#divSchemes=[ 'linear', 'cubic', 'downwind', 'midPoint', 'MUSCL', 'Minmod', 'QUICK', 
#		'filteredLinear', 'upwind', 'vanLeer']

ddtSchemes=['Euler','backward','CrankNicholson 1.0']  # currently not used

# set original directory to top of tree
origdir=os.getcwd()

# Run everything from results directory & create figures directory for holding all plots
if os.path.isdir('figures'):    	# does figures directory exist?
	shutil.rmtree('figures')	# yes, remove old directory
os.mkdir('figures')                     # create figures dir
if os.path.isdir('results'):            # does results directory exist?
	shutil.rmtree('results')	# yes, remove old directory
os.mkdir('results')                     # create results dir
os.chdir('results')

caseDirs = []

# Loop over available divSchemes
for divScheme in divSchemes:

	if os.path.isdir(divScheme):    	# does directory named dir exist?
		shutil.rmtree(divScheme)	# yes, remove old directory
	os.mkdir(divScheme)
	os.chdir(divScheme)

        # Loop over grid refinement:  baseline is coarse mesh of 8 x 8
		
	meshDir = ['mesh8','mesh16','mesh32','mesh64']
	#meshDir = ['mesh32']
	maxError = {}
	aveError = {}
	gridSize= [8.0, 4.0, 2.0, 1.0]
	#gridSize= [2.0]

	# Loop over chosen mesh densities
	for mesh in meshDir:
				
		if os.path.isdir(mesh):		# does the mesh directory exist?
			shutil.rmtree(mesh)      # yes, remove old directory

		# cd into mesh directory
		os.mkdir(mesh)
		os.chdir(mesh)			

		# Loop over chosen nCorrector values
		for nCorrector in nCorrectors:
			dirName = 'nCorrectors' + nCorrector
			if os.path.isdir(dirName):		# does the nCorrector directory exist?
				shutil.rmtree(dirName)      # yes, remove old directory

			# cd into nCorrector directory
			os.mkdir(dirName)
			os.chdir(dirName)							

			# Loop over chosen time step values
			for timeStep in timeSteps:

				dirName = 'timeStep' + timeStep
				if os.path.isdir(dirName):		# does the time step directory exist?
					shutil.rmtree(dirName)      # yes, remove old directory

				# cd into timeStep directory
				os.mkdir(dirName)
				os.chdir(dirName)		
				
				# Loop over chosen relaxation factor values
				for factors in relaxationFactors:
					relaxationDir = 'relaxationsU' + factors[0] + 'p'+ factors[1]
					caseDir = divScheme+"/"+mesh+"/nCorrectors"+nCorrector+"/timeStep"+timeStep+"/"+relaxationDir
					caseDirs.append(caseDir)
					#if os.path.isdir(relaxationDir):		# does the time step directory exist?
						#shutil.rmtree(relaxationDir)      # yes, remove old directory

					# cd into case directory
					print
					print 'Executing',caseDir 
					meshIndex = meshDir.index(mesh) 

					# use pyFoam to clone the baseline case
					print '   pyFoamCloneCase.py' 
					cmd='pyFoamCloneCase.py ../../../../../baseline %s' % relaxationDir
					pipefile = open('output', 'w')
					retcode = call(cmd,shell=True,stdout=pipefile,stderr=pipefile)
					pipefile.close()
					os.remove('output')
					#os.chdir('../../../../../../baseline')
					
					# cd into case directory
					os.chdir(relaxationDir)		
					
					# change the divScheme for "div(phi,U)  Gauss linear;" in the fvSchemes file
					infilename='system/fvSchemes'
					outfilename='system/fvSchemesTemp'
					ifile = open(infilename, 'r')
					ofile = open(outfilename, 'w')
					lines = ifile.readlines()
					for line in lines:
						words = line.split()
						for word in words:
							index =words.index(word) 
							if word == 'div(phi,U)': 
								if divScheme == 'limitedLinearV':
									words[index+2] = divScheme+' 1;'
								else:
									words[index+2] = divScheme+';'
								continue
						newline=' '.join(words)
						ofile.write('%s\n' % newline)
					ifile.close()
					ofile.close()
					os.remove(infilename)
					os.rename(outfilename,infilename)

					# Modify the fvSolutions file: Change relaxation factors and nCorrectors value
					infilename='system/fvSolution'
					outfilename='system/fvSolutionTemp'
					ifile = open( infilename, 'r')
					ofile = open(outfilename, 'w')
					lines = ifile.readlines()
					T=False
					for line in lines:
						words = line.split()
						try:
							if words[0]=='relaxationFactors':	
								T=True	# Find out when the relaxation dictionary of the file has been reached
							if words[0] == '}': # and only after that allow the values to be changed
								T=False
						except IndexError:
							pass
						for word in words:
							index =words.index(word) 
							if word == 'U' and T: 
								words[index+1] = factors[0]+';'
							if word == 'p' and T: 
								words[index+1] = factors[1]+';'
							if word == 'nCorrectors': 
								words[index+1] = nCorrector+';'
	
						newline=' '.join(words)
						ofile.write('%s\n' % newline)
					ifile.close()
					ofile.close()
					os.remove(infilename)
					os.rename(outfilename,infilename)


					#Modify the controlDict file: Change the time step (deltaT)
					infilename='system/controlDict'
					outfilename='system/controlDictTemp'
					ifile = open( infilename, 'r')
					ofile = open(outfilename, 'w')
					lines = ifile.readlines()
					for line in lines:
						words = line.split()
						for word in words:
							index =words.index(word) 
							if word == 'deltaT': 
								words[index+1] = timeStep+';'
								continue
						newline=' '.join(words)
						ofile.write('%s\n' % newline)
					ifile.close()
					ofile.close()
					os.remove(infilename)
					os.rename(outfilename,infilename)


					
					# set block size in blockMeshDict
					dict="constant/polyMesh/blockMeshDict"  	# blockMeshDict
					bm=BlockMesh(dict)                              # define bm to be a pyFoam BlockMesh object 
					number=meshDir.index(mesh)                      # find index of current mesh: 0, 1, 2, 3
					bm.refineMesh((2**number,2**number,1))         	# refineMesh by factor of 2^number over baseline
					meshDim = 8*2**number

					# run blockMesh
					print '   blockMesh'
					cmd='blockMesh'
					pipefile = open('output', 'w')
					retcode = call(cmd,shell=True,stdout=pipefile)
					pipefile.close()
					os.remove('output')

					# run funkySetFields to set initial conditions 
					print '   funkySetFields -time 0'
					cmd='funkySetFields -time 0'
					pipefile = open('output', 'w')
					retcode = call(cmd,shell=True,stdout=pipefile)
					pipefile.close()
					os.remove('output')

					# run transientSimpleFoam to solve Navier Stokes equations
					print '   transientSimpleFoam > log 2>&1'
					cmd='transientSimpleFoam > log 2>&1'
					pipefile = open('output', 'w')
					retcode = call(cmd,shell=True,stdout=pipefile)
					pipefile.close()
					os.remove('output')

					# run custom post-processor for computing analytical solution at each time step, 
					# and for computing comparison error
					print '   analyticalSolutionTaylorVortex -time 1'
					cmd='analyticalSolutionTaylorVortex -time 1'
					pipefile = open('ASTVOutput', 'w')
					retcode = call(cmd,shell=True,stdout=pipefile)
					pipefile.close()

					# extract maximum and average error at endTime (time = 1) from STDOUT
					pipefile = open('ASTVOutput', 'r')
					lines = pipefile.readlines()
					for line in lines:
						words = line.split()
						for word in words:
							if word == 'Maximum': 
								value=float(words[len(words)-1])
								try:# Create a list containing the gridSize-maximum error pairs [[8,value],[4,value],[2,value],[1,value]] for each case
									maxError['nC'+nCorrector+',tS'+timeStep+','+relaxationDir].append([gridSize[meshIndex],value])
								except KeyError:
									maxError['nC'+nCorrector+',tS'+timeStep+','+relaxationDir]=[[gridSize[meshIndex],value]]
							if word == 'Average': 
								value=float(words[len(words)-1])
								try:
									aveError['nC'+nCorrector+',tS'+timeStep+','+relaxationDir].append([gridSize[meshIndex],value])
								except KeyError:
									aveError['nC'+nCorrector+',tS'+timeStep+','+relaxationDir]=[[gridSize[meshIndex],value]]
					pipefile.close()

					# run sample utility
					print '   sample -time 1'
					cmd='sample -time 1'
					pipefile = open('output', 'w')
					errorfile = open('sampleError', 'w')
					retcode = call(cmd,shell=True,stdout=pipefile,stderr=errorfile)
					pipefile.close()
					errorfile.close()
					if not bool(os.path.getsize('sampleError')): # if the errorfile is empty
						os.remove('sampleError')			# remove it
					os.remove('output')

					# construct contour & vector plots
					print '   python '+'/tools/plotContours.py '
					cmd='python '+origdir+'/tools/plotContours.py '+ str(meshDim)+' '+divScheme+' '+mesh+' '+nCorrector+' '+timeStep+' '+relaxationDir+' '+origdir
					retcode = call(cmd,shell=True)
					print '   python '+'/tools/plotVectors.py '
					cmd='python '+origdir+'/tools/plotVectors.py '+ str(meshDim)+' '+divScheme+' '+mesh+' '+nCorrector+' '+timeStep+' '+relaxationDir+' '+origdir
					retcode = call(cmd,shell=True)

					#move.eps image files to figures directory
					filelist=glob.glob('*.eps')
					for file in filelist:
						shutil.move(file, origdir+"/figures/.")


					os.chdir('..')
				os.chdir('..')
			os.chdir('..')
		os.chdir("..")



	# save data for error vs. grid resolution-plots into temporary files
	keys = maxError.keys()
	for key in keys:
		ofile = open(str(key)+'-error', 'w')
		for i in range(len(maxError[key])):
			out=[str(maxError[key][i][0]),str(maxError[key][i][1]),str(aveError[key][i][1])]
			newline=' '.join(out)
			ofile.write('%s\n' % str(newline))
        ofile.close()

	os.chdir("..")


print
print
# Plot U and p on lines x=0 and y=0
print '/tools/plotLines.py '
cases=caseDirs[0]
for i in range(1,len(caseDirs)):
	cases+="-"+caseDirs[i]
cmd='python '+origdir+'/tools/collectLinesData.py '+cases
retcode = call(cmd,shell=True)
dataList = glob.glob('*mesh8*_temp')
for file in dataList:
	cmd='python '+origdir+'/tools/plotLines.py '+file
	retcode = call(cmd,shell=True)
filelist=glob.glob('*.eps')
for file in filelist:
	shutil.move(file, origdir+"/figures/.")
removeCommand='rm -rf *temp'
call(removeCommand,shell=True)

# Plot maximum velocity errors against mesh density
print '/tools/plotError.py'
cmd='python '+origdir+'/tools/plotError.py'
retcode = call(cmd,shell=True)
filelist=glob.glob('*.eps')
for file in filelist:
	shutil.move(file, origdir+"/figures/.")

# Create an appendix with chosen pictures
print '/tools/appendix.py'
cmd='python '+origdir+'/tools/appendix.py'
retcode = call(cmd,shell=True)


