/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2008 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    analyticalSolutionTaylorVortex

Description
    Generates an analytical solution for the Taylor-Green Vortex, e.g.,  
    http://en.wikipedia.org/wiki/Taylor-Green_vortex

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

    timeSelector::addOptions();
#   include "setRootCase.H"
#   include "createTime.H"
    instantList timeDirs = timeSelector::select0(runTime, args);
#   include "createMesh.H"
#   include "createFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
    forAll(timeDirs, timeI)
    {
        runTime.setTime(timeDirs[timeI], timeI);

    Info << "\nEvaluating analytical solution for time =" << runTime.timeName() << endl;

    dimensionedScalar xx ("xx",dimensionSet(0,-1,0,0,0,0,0),scalar(1));
    volScalarField centresX = xx*UA.mesh().C().component(0);
    volScalarField centresY = xx*UA.mesh().C().component(1);
    scalar time = runTime.value();
    dimensionedScalar coeff1("coeff1",dimensionSet(0,1,-1,0,0,0,0), Foam::exp(scalar(-2.0*time)));
    dimensionedScalar coeff2("coeff2",dimensionSet(0,2,-2,0,0,0,0), Foam::exp(scalar(-4.0*time)));

    volScalarField uax = -cos(centresX)*sin(centresY)*coeff1;
    volScalarField uay =  sin(centresX)*cos(centresY)*coeff1;
    UA = uax*vector(1,0,0) + uay*vector(0,1,0);

    pA = -0.25*(cos(2.*centresX) + cos(2.*centresY))*coeff2;
    
    UA.write();
    pA.write();

    volScalarField errorVelMag
      (
         IOobject
	 (
	    "errorVelMag",
            runTime.timeName(),
            mesh,
            IOobject::NO_READ
          ),
          mag(UA)-mag(U)
      );

      Info<< "     Maximum error" << max(errorVelMag)/max(mag(UA)) << endl;
      Info<< "     Average error" << errorVelMag.weightedAverage(mesh.V())/max(mag(UA)) << endl;
      volScalarField sqrError = sqr(errorVelMag); 
      Info<< "     RMS error    " << sqrt(sqrError.weightedAverage(mesh.V()))/max(mag(UA)) << endl;

    }

    return(0);
}

// ************************************************************************* //
