from subprocess import call
import sys, glob
import os

# This script will produce by default a .pdf file that is identical to the appendix in the 
# original study. If the parameters in computation are changed, a pdf with lineError.eps-figures
# from computations with the 32x32 cells-grid and the errorConvergence.eps is generated.

def makeCaption(file):
	#file='pErrorQUICKmesh32nCorrectors80timeStep0.05relaxationsU0.8p0.3.eps'
	file=file.split('.eps') 
	caseName=file[0].split('pError'); 
	caseName=caseName[1] 

	divScheme, tail = caseName.split('mesh')
	mesh, tail = tail.split('nCorrectors')
	nC, tail = tail.split('timeStep')
	timeStep, tail = tail.split('relaxationsU')
	alphaU, alphap = tail.split('p')
		
	figTitle = "\n div(phi,U)="+divScheme + ", mesh=("+mesh+"x"+mesh+")"+", nCorrectors=" + nC +", "+ r"$\Delta t$=" +timeStep+'\n'
	figTitle += ", " + r"$\alpha_U$=" + alphaU + ", " + r"$\alpha_p$=" + alphap
	return caseName, figTitle


os.chdir('../')
caseDir = os.getcwd()
os.chdir(caseDir+'/figures')

linesList_temp = glob.glob('*lines*'); linesList=['']*len(linesList_temp)
pErrorList_temp = glob.glob('*pError*mesh32*'); pErrorList=['']*len(pErrorList_temp)
UMagErrorList_temp = glob.glob('*UMagError*mesh32*'); UMagErrorList=['']*len(UMagErrorList_temp)

try:
	for file in linesList_temp:
		if '30' in file:
			if '0.05' in file:
				if '0.7' in file:
					linesList[0]=file
				else:
					linesList[1]=file
			else:
				if '0.7' in file:
					linesList[2]=file
				else:
					linesList[3]=file
		else:
			if '0.05' in file:
				if '0.7' in file:
					linesList[4]=file
				else:
					linesList[5]=file
			else:
				if '0.7' in file:
					linesList[6]=file
				else:
					linesList[7]=file
except IndexError:
	linesList=linesList_temp
	pErrorList=pErrorList_temp
	UMagErrorList=UMagErrorList_temp

for i in range(len(linesList)):
	caseName, caption = makeCaption(pErrorList_temp[i])
	for j in range(len(pErrorList_temp)):
		if caseName in pErrorList_temp[j]:
			pErrorList[i] = pErrorList_temp[j]
		if caseName in UMagErrorList_temp[j]:
			UMagErrorList[i] = UMagErrorList_temp[j]

os.chdir(caseDir+'/figures')

linesList = glob.glob('*lines*')
pErrorList = glob.glob('*pError*mesh32*')
UMagErrorList = glob.glob('*UMagError*mesh32*')

os.chdir(caseDir+'/latex/')

app = open('appendixSrc.tex','w')
app.write('\\thispagestyle{empty} \n')
app.write('\\begin{figure}[h!] \n')
app.write('\\centering \n')
app.write('\\subfloat{\label{}}\\resizebox{100mm}{78mm}{\includegraphics*{'+caseDir+'/latex/errorConvergence.eps}}  \n')
app.write('\\subfloat{\label{}}\\resizebox{140mm}{78mm}{\includegraphics*{'+caseDir+'/figures/errorConvergence.eps}} \n')
app.write('\\caption{Convergence of Spatial and Temporal Errors with \\textit{transientSimpleFoam} and icoFoam. \\textit{icoFoam} in the upper and \\textit{transientSimpleFoam} in the lower picture}  \n')
app.write('\\label{fig:spatialTemporalError} \n')
app.write('\\end{figure} \n')
app.write('\\thispagestyle{empty} \n')
app.write('\\begin{figure} \n')
app.write('\\centering \n')
app.write('\\centering \\resizebox{100mm}{80mm}{\includegraphics*%[0mm,10mm][185mm,130mm] \n')
app.write('{linesQUICK.eps}}')
app.write('\\caption{div(phi,U)=QUICK, mesh (32x32), \\textit{icoFoam}} \n')
app.write('\\label{fig:linesIco} \n'); 
app.write('\\end{figure} \n')
app.write('\\clearpage \n')


app.write('\\clearpage')
for i in range(len(pErrorList)):
	caseName, caption = makeCaption(pErrorList[i])
	if i == 0:
		label = 'fig:first'
	elif i == len(pErrorList)-1:
		label = 'fig:last'
	else:
		label = ''
	app.write('\\thispagestyle{empty} \n')
	app.write('\\begin{figure} \n')
	app.write('\\centering \n')
	fileName = linesList[i]
	app.write('\\centering \\resizebox{100mm}{80mm}{\includegraphics*%[0mm,10mm][185mm,130mm] \n')
	app.write('{'+caseDir+'/figures/'+fileName+'}}')
	app.write('\\caption{'+caption+'} \n')
	app.write('\\label{'+label+'} \n'); 
	app.write('\\end{figure}')
	app.write(' \n')
	app.write(' \n')
	#app.write('\\clearpage')
app.close()

# Compile the pdf
call('latex appendix.tex',shell=True)
call('dvips appendix.dvi -t a4 -o appendix.ps',shell=True)
call('ps2pdf appendix.ps appendix.pdf',shell=True)

