#!/opt/local/bin/python 
# set matplotlib backend to PDF for fast off-screen plotting (this has to be done before loadtxting pylab)
import matplotlib
matplotlib.use('ps')
#matplotlib.use('PDF')

# now loadtxt pylab and other modules
from pylab import *
import sys, os
import glob


# get command line arguments
meshDim = float(sys.argv[1])
divScheme = sys.argv[2]
mesh = sys.argv[3]
nCorrectors = sys.argv[4]
timeStep = sys.argv[5]
relaxation = sys.argv[6] # = relaxationsU0.8p0.3 
origdir = sys.argv[7]
relaxations = relaxation.split('relaxationsU')
alphaU, alphap = relaxations[1].split('p')
# contours
ua=loadtxt("surfaces/1/UA_plane.raw")
us=loadtxt("surfaces/1/U_plane.raw")

envDict = os.environ
OFv = envDict['WM_PROJECT_VERSION'][:3]#Take 3 first characters (eg. 1.7.1->1.7 )
if float(OFv) > 1.5:
	x_all=ua[:,0];   x = np.array(list(x_all[2*i] for i in range(len(x_all)/2))) 
	y_all=ua[:,1];   y = np.array(list(y_all[2*i] for i in range(len(y_all)/2)))
	uA_all=ua[:,3];  uA = np.array(list(uA_all[2*i] for i in range(len(uA_all)/2)))
	vA_all=ua[:,4];  vA = np.array(list(vA_all[2*i] for i in range(len(vA_all)/2)))
	uS_all=us[:,3]; uS = np.array(list(uS_all[2*i] for i in range(len(uS_all)/2)))
	vS_all=us[:,4];  vS = np.array(list(vS_all[2*i] for i in range(len(vS_all)/2)))
else:
	x=ua[:,0]
	y=ua[:,1]
	uA=ua[:,3]
	vA=ua[:,4]
	uS=us[:,3]
	vS=us[:,4]

X=reshape(x,(meshDim,meshDim))
Y=reshape(y,(meshDim,meshDim))
UA=reshape(sqrt(uA**2+vA**2),(meshDim,meshDim))
US=reshape(sqrt(uS**2+vS**2),(meshDim,meshDim))
levels = arange(0, 1.05*max(UA.max(),US.max()), 0.01)

figure(figsize=(8,8))
axis('scaled')
title("Analytical solution $(A)$ Velocity Magnitude")
xlim(-1.6,1.6)
ylim(-1.6,1.6)
contourf(X,Y,UA,levels)
colorbar(fraction=0.045)
#Check if the analytical case has already been plotted
curDir=os.getcwd()
os.chdir(origdir+"/figures/")
T=bool(glob.glob("UMagAnalytical"+mesh+".eps"))
os.chdir(curDir)
if not T: # if not, plot it
	savefig("UMagAnalytical"+mesh+".eps",format='eps')
	#savefig("UMagAnalytical"+mesh+".pdf",dpi=(300),format='pdf')
clf()

figure(figsize=(8,8))
axis('scaled')
title("Simulation $(S)$ Velocity Magnitude,\n div(phi,U)="+divScheme+", nC="+nCorrectors+",\n"+ r"$\Delta t$="+timeStep+", " + r"$\alpha_U$=" + alphaU + ", " + r"$\alpha_p$=" + alphap)
xlim(-1.6,1.6)
ylim(-1.6,1.6)
contourf(X,Y,US,levels)
colorbar(fraction=0.045)
savefig("UMagSimulation"+divScheme+mesh+"nCorrectors"+nCorrectors+"timeStep"+timeStep+relaxation+".eps",format='eps')
#savefig("UMagSimulation"+divScheme+mesh+"nCorrectors"+nCorrectors+"timeStep"+timeStep+relaxation+".pdf",dpi=(300),format='pdf')
clf()

figure(figsize=(8,8))
axis('scaled')
title("\% Comparison error $(S-A)/A_{max}$,\n div(phi,U)="+divScheme+", nC="+nCorrectors+",\n"+ r"$\Delta t$="+timeStep+", " + r"$\alpha_U$=" + alphaU + ", " + r"$\alpha_p$=" + alphap)
CE=US-UA
xlim(-1.6,1.6)
ylim(-1.6,1.6)
contourf(X,Y,100*CE/UA.max(),20)
colorbar(fraction=0.045)
savefig("UMagError"+divScheme+mesh+"nCorrectors"+nCorrectors+"timeStep"+timeStep+relaxation+".eps",format='eps')
#savefig("UMagError"+divScheme+mesh+"nCorrectors"+nCorrectors+"timeStep"+timeStep+relaxation+".pdf",dpi=(300),format='pdf')
clf()

pa=loadtxt("surfaces/1/pA_plane.raw")
p=loadtxt("surfaces/1/p_plane.raw")
if float(OFv) > 1.5:
	x_all=pa[:,0];   x = np.array(list(x_all[2*i] for i in range(len(x_all)/2)))
	y_all=pa[:,1];   y = np.array(list(y_all[2*i] for i in range(len(y_all)/2)))
	pA_all=pa[:,3];  pA = np.array(list(pA_all[2*i] for i in range(len(pA_all)/2)))
	pS_all=p[:,3];   pS = np.array(list(pS_all[2*i] for i in range(len(pS_all)/2)))
else:
	x=pa[:,0]
	y=pa[:,1]
	pA=pa[:,3]
	pS=p[:,3]

X=reshape(x,(meshDim,meshDim))
Y=reshape(y,(meshDim,meshDim))
pA=reshape(pA,(meshDim,meshDim))
pS=reshape(pS,(meshDim,meshDim))

figure(figsize=(8,8))
axis('scaled')
title("Analytical solution $(A)$ pressure")
xlim(-1.6,1.6)
ylim(-1.6,1.6)
contourf(X,Y,pA,20)
colorbar(fraction=0.045)
#Check if the analytical case has already been plotted
curDir=os.getcwd()
os.chdir(origdir+"/figures/")
T=bool(glob.glob("pAnalytical"+mesh+".eps"))
os.chdir(curDir)
if not T: # if not, plot it
	savefig("pAnalytical"+mesh+".eps",format='eps')
	#savefig("pAnalytical"+mesh+".pdf",dpi=(300),format='pdf')

clf()

figure(figsize=(8,8))
axis('scaled')
title("Simulation $(S)$ pressure,\n div(phi,U)="+divScheme+", nC="+nCorrectors+",\n"+ r"$\Delta t$="+timeStep+", " + r"$\alpha_U$=" + alphaU + ", " + r"$\alpha_p$=" + alphap)
xlim(-1.6,1.6)
ylim(-1.6,1.6)
contourf(X,Y,pS,20)
colorbar(fraction=0.045)
savefig("pSimulation"+divScheme+mesh+"nCorrectors"+nCorrectors+"timeStep"+timeStep+relaxation+".eps",format='eps')
#savefig("pSimulation"+divScheme+mesh+"nCorrectors"+nCorrectors+"timeStep"+timeStep+relaxation+".pdf",dpi=(300),format='pdf')
clf()

figure(figsize=(8,8))
axis('scaled')
title("\% Comparison error $(S-A)/A_{max}$ for pressure,\n div(phi,U)="+divScheme+", nC="+nCorrectors+",\n"+ r"$\Delta t$="+timeStep+", " + r"$\alpha_U$=" + alphaU + ", " + r"$\alpha_p$=" + alphap)
CE=pS-pA
xlim(-1.6,1.6)
ylim(-1.6,1.6)
contourf(X,Y,100*CE/UA.max(),20)
colorbar(fraction=0.045)
savefig("pError"+divScheme+mesh+"nCorrectors"+nCorrectors+"timeStep"+timeStep+relaxation+".eps",format='eps')
#savefig("pError"+divScheme+mesh+"nCorrectors"+nCorrectors+"timeStep"+timeStep+relaxation+".pdf",dpi=(300),format='pdf')
clf()

